
CREATE TABLE cache (
  query VARCHAR(128) NOT NULL,
  value MEDIUMBLOB,
  stamp TIMESTAMP,
  index(query)
  );

CREATE TABLE control (
  id VARCHAR(128) NOT NULL,
  class VARCHAR(64) NOT NULL,
  count INT UNSIGNED,
  stamp TIMESTAMP,
  index(id)
  );

